@@ -1,5 +1,5 @@
 /*
- * The IKE Scanner (ike-scan) is Copyright (C) 2003-2004 Roy Hills,
+ * The IKE Scanner (ike-scan) is Copyright (C) 2003-2005 Roy Hills,
  * NTA Monitor Ltd.
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: error.c,v 1.13 2004/01/02 09:30:36 rsh Exp $
+ * $Id: error.c,v 1.15 2005/01/01 16:49:53 rsh Exp $
  *
  * error.c -- error routines for IKE Scanner (ike-scan)
  *
@@ -33,7 +33,7 @@
 
 #include "ike-scan.h"
 
-static char rcsid[] = "$Id: error.c,v 1.13 2004/01/02 09:30:36 rsh Exp $"; /* RCS ID for ident(1) */
+static char rcsid[] = "$Id: error.c,v 1.15 2005/01/01 16:49:53 rsh Exp $"; /* RCS ID for ident(1) */
 
 int daemon_proc;	/* Non-zero if process is a daemon */
 
@@ -128,5 +128,5 @@ err_print (int errnoflag, int level, const char *fmt, va_list ap) {
 
 void
 error_use_rcsid(void) {
-   printf("%s\n", rcsid);       /* Use rcsid to stop compiler optimising away */
+   fprintf(stderr, "%s\n", rcsid);       /* Use rcsid to stop compiler optimising away */
 }
