@@ -1,5 +1,5 @@
 /*
- * The IKE Scanner (ike-scan) is Copyright (C) 2003-2004 Roy Hills,
+ * The IKE Scanner (ike-scan) is Copyright (C) 2003-2005 Roy Hills,
  * NTA Monitor Ltd.
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: isakmp.c,v 1.23 2004/01/10 18:22:52 rsh Exp $
+ * $Id: isakmp.c,v 1.32 2005/01/01 16:49:53 rsh Exp $
  *
  * Author: Roy Hills
  * Date: 7 November 2003
@@ -33,9 +33,10 @@
 
 #include "ike-scan.h"
 
-static char rcsid[] = "$Id: isakmp.c,v 1.23 2004/01/10 18:22:52 rsh Exp $";	/* RCS ID for ident(1) */
+static char rcsid[] = "$Id: isakmp.c,v 1.32 2005/01/01 16:49:53 rsh Exp $";	/* RCS ID for ident(1) */
 
-extern int experimental;
+extern int experimental_flag;
+extern struct psk_crack psk_values;
 
 /*
  *	make_isakmp_hdr -- Construct an ISAKMP Header
@@ -631,6 +632,41 @@ make_id(size_t *length, unsigned next, unsigned idtype, unsigned char *id_data,
    return payload;
 }
 
+/*
+ *      make_udphdr -- Construct a UDP header for encapsulated IKE
+ *
+ *      Inputs:
+ *
+ *      length  (output) length of UDP header
+ *      sport           UDP source port
+ *      dport           UDP destination port
+ *      udplen          UDP length
+ *
+ *      Returns:
+ *
+ *      Pointer to constructed UDP header
+ *
+ *      This constructs a UDP header which is used for IKE
+ *      encapsulated within TCP.
+ */
+unsigned char*
+make_udphdr(size_t *length, int sport, int dport, unsigned udplen) {
+   unsigned char *payload;
+   struct ike_udphdr *hdr;
+
+   payload = Malloc(sizeof(struct ike_udphdr));
+   hdr = (struct ike_udphdr*) payload; /* Overlay UDP hdr on payload */
+
+   hdr->source = htons(sport);
+   hdr->dest   = htons(dport);
+   hdr->len    = htons(udplen);
+   hdr->check  = 0; /* should use in_cksum() */
+
+   *length = sizeof(struct ike_udphdr);
+
+   return payload;
+}
+
 /*
  *	skip_payload -- Skip an ISAMKP payload
  *
@@ -870,7 +906,10 @@ process_attr(unsigned char **cp, size_t *len) {
       NULL,
       "MD5",					/*  1 */
       "SHA1",					/*  2 */
-      "Tiger"					/*  3 */
+      "Tiger",					/*  3 */
+      "SHA2-256",				/*  4 */
+      "SHA2-384",				/*  5 */
+      "SHA2-512"				/*  6 */
    };
    static const char *auth_names[] = {		/* From RFC 2409 App. A */
       NULL,
@@ -878,7 +917,10 @@ process_attr(unsigned char **cp, size_t *len) {
       "DSS",					/*  2 */
       "RSA_Sig",				/*  3 */
       "RSA_Enc",				/*  4 */
-      "RSA_RevEnc"				/*  5 */
+      "RSA_RevEnc",				/*  5 */
+      "ElGamel_Enc",				/*  6 */
+      "ElGamel_RevEnc",				/*  7 */
+      "ECDSA_Sig"				/*  8 */
    };
    static const char *dh_names[] = {		/* From RFC 2409 App. A */
       NULL,					/* and RFC 3526 */
@@ -887,14 +929,14 @@ process_attr(unsigned char **cp, size_t *len) {
       "3:ec2n155",				/*  3 */
       "4:ec2n185",				/*  4 */
       "5:modp1536",				/*  5 */
-      NULL,					/*  6 */
-      NULL,					/*  7 */
-      NULL,					/*  8 */
-      NULL,					/*  9 */
-      NULL,					/* 10 */
-      NULL,					/* 11 */
-      NULL,					/* 12 */
-      NULL,					/* 13 */
+      "6:ec2n163",				/*  6 */
+      "7:ec2n163",				/*  7 */
+      "8:ec2n283",				/*  8 */
+      "9:ec2n283",				/*  9 */
+      "10:ec2n409",				/* 10 */
+      "11:ec2n409",				/* 11 */
+      "12:ec2n571",				/* 12 */
+      "13:ec2n571",				/* 13 */
       "14:modp2048",				/* 14 */
       "15:modp3072",				/* 15 */
       "16:modp4096",				/* 16 */
@@ -1212,7 +1254,209 @@ process_id(unsigned char *cp, size_t len) {
    return msg;
 }
 
+/*
+ *	print_payload -- Print an ISAKMP payload in hex
+ *
+ *	Inputs:
+ *
+ *	cp	Pointer to start of ISAKMP payload
+ *	payload	Numeric value of this payload type, 0 = ISAKMP header
+ *	dir	Direction: 'I' for initiator or 'R' for responder
+ *
+ *	Returns:
+ *
+ *	None
+ *
+ *	This function is used for debugging.  It trusts the length in the
+ *	generic ISAKMP header, and so could misbehave with corrupted packets.
+ */
+void
+print_payload(unsigned char *cp, unsigned payload, int dir) {
+   struct isakmp_generic *hdr = (struct isakmp_generic *) cp;
+   struct isakmp_hdr *ihdr = (struct isakmp_hdr *) cp;
+   char *hexdata;
+   unsigned char *data;
+   size_t data_len;
+
+   if (payload) {	/* Some other payload */
+      data = cp + sizeof(struct isakmp_generic);  /* Points to start of data */
+      data_len = ntohs(hdr->isag_length);
+      data_len -= sizeof(struct isakmp_generic);
+      hexdata = hexstring(data, data_len);
+      switch (payload) {
+         case ISAKMP_NEXT_SA:
+            printf("sa%c_b_hex=\"%s\"\n", (dir=='I')?'i':'r', hexdata);
+            break;
+         case ISAKMP_NEXT_KE:
+            printf("g_x%c_hex=\"%s\"\n", (dir=='I')?'i':'r', hexdata);
+            break;
+         case ISAKMP_NEXT_ID:
+            printf("idi%c_b_hex=\"%s\"\n", (dir=='I')?'i':'r', hexdata);
+            break;
+         case ISAKMP_NEXT_HASH:
+            printf("expected_hash_%c_hex=\"%s\"\n", (dir=='I')?'i':'r', hexdata);
+            break;
+         case ISAKMP_NEXT_NONCE:
+            printf("n%c_b_hex=\"%s\"\n", (dir=='I')?'i':'r', hexdata);
+            break;
+         default:
+            printf("UNKNOWN PAYLOAD TYPE: %d\n", payload);
+            break;
+      }
+      free(hexdata);
+   } else {	/* ISAKMP Header */
+      hexdata = hexstring((unsigned char *)ihdr->isa_icookie, 8);
+      printf("cky_i_hex=\"%s\"\n", hexdata);
+      free(hexdata);
+      hexdata = hexstring((unsigned char *)ihdr->isa_rcookie, 8);
+      printf("cky_r_hex=\"%s\"\n", hexdata);
+      free(hexdata);
+   }
+}
+
+/*
+ *	add_psk_crack_payload -- Add an ISAKMP payload to PSK crack structure
+ *
+ *	Inputs:
+ *
+ *	cp	Pointer to start of ISAKMP payload
+ *	payload	Numeric value of this payload type, 0 = ISAKMP header
+ *	dir	Direction: 'I' for initiator or 'R' for responder
+ *
+ *	Returns:
+ *
+ *	None
+ *
+ *	This function is used for debugging.  It trusts the length in the
+ *	generic ISAKMP header, and so could misbehave with corrupted packets.
+ */
+void
+add_psk_crack_payload(unsigned char *cp, unsigned payload, int dir) {
+   struct isakmp_generic *hdr = (struct isakmp_generic *) cp;
+   struct isakmp_hdr *ihdr = (struct isakmp_hdr *) cp;
+   unsigned char *data;
+   size_t data_len;
+
+   if (payload) {	/* Some other payload */
+      data_len = ntohs(hdr->isag_length) - sizeof(struct isakmp_generic);
+      data = Malloc(data_len);
+      memcpy(data, cp + sizeof(struct isakmp_generic), data_len);
+
+      switch (payload) {
+         case ISAKMP_NEXT_SA:
+            if (dir == 'I') {
+               psk_values.sai_b = data;
+               psk_values.sai_b_len = data_len;
+            }
+            break;
+         case ISAKMP_NEXT_KE:
+            if (dir == 'I') {
+               psk_values.g_xi = data;
+               psk_values.g_xi_len = data_len;
+            } else {
+               psk_values.g_xr = data;
+               psk_values.g_xr_len = data_len;
+            }
+            break;
+         case ISAKMP_NEXT_ID:
+            if (dir == 'R') {
+               psk_values.idir_b = data;
+               psk_values.idir_b_len = data_len;
+            }
+            break;
+         case ISAKMP_NEXT_HASH:
+            if (dir == 'R') {
+               psk_values.hash_r = data;
+               psk_values.hash_r_len = data_len;
+            }
+            break;
+         case ISAKMP_NEXT_NONCE:
+            if (dir == 'I') {
+               psk_values.ni_b = data;
+               psk_values.ni_b_len = data_len;
+            } else {
+               psk_values.nr_b = data;
+               psk_values.nr_b_len = data_len;
+            }
+            break;
+         default:
+            warn_msg("add_psk_crack_payload: UNKNOWN PAYLOAD TYPE: %d\n",
+                     payload);
+            break;
+      }
+   } else {	/* ISAKMP Header */
+      data_len=8;	/* ISAKMP cookies are 8 bytes long */
+      data=Malloc(data_len);
+      memcpy(data, (unsigned char *)ihdr->isa_rcookie, 8);
+      psk_values.cky_r = data;
+      psk_values.cky_r_len = data_len;
+
+      data=Malloc(data_len);
+      memcpy(data, (unsigned char *)ihdr->isa_icookie, 8);
+      psk_values.cky_i = data;
+      psk_values.cky_i_len = data_len;
+   }
+}
+
+/*
+ *	print_psk_crack_values -- Display the PSK crack values
+ *
+ *	Inputs:
+ *
+ *	psk_crack_file	Name of PSK data output file, or NULL for stdout
+ *
+ *	Returns:
+ *
+ *	None
+ *
+ *	This function prints the PSK crack values in the following format:
+ *
+ *	g_xr:g_xi:cky_r:cky_i:sai_b:idir_b:ni_b:nr_b:hash_r
+ */
+void
+print_psk_crack_values(const char *psk_crack_file) {
+   char *hexdata;
+   FILE *fp;
+
+   if (psk_crack_file[0]) {
+      if ((fp = fopen(psk_crack_file, "w")) == NULL) {
+         err_sys("ERROR: fopen");
+      }
+   } else {
+      fp = stdout;
+      printf("IKE PSK parameters (g_xr:g_xi:cky_r:cky_i:sai_b:idir_b:ni_b:nr_b:hash_r):\n");
+   }
+
+   hexdata=hexstring(psk_values.g_xr, psk_values.g_xr_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.g_xi, psk_values.g_xi_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.cky_r, psk_values.cky_r_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.cky_i, psk_values.cky_i_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.sai_b, psk_values.sai_b_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.idir_b, psk_values.idir_b_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.ni_b, psk_values.ni_b_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.nr_b, psk_values.nr_b_len);
+   fprintf(fp, "%s:", hexdata);
+   free(hexdata);
+   hexdata=hexstring(psk_values.hash_r, psk_values.hash_r_len);
+   fprintf(fp, "%s\n", hexdata);
+   free(hexdata);
+}
+
 void
 isakmp_use_rcsid(void) {
-   printf("%s\n", rcsid);	/* Use rcsid to stop compiler optimising away */
+   fprintf(stderr, "%s\n", rcsid);	/* Use rcsid to stop compiler optimising away */
 }
