@@ -1,5 +1,5 @@
 /*
- * The IKE Scanner (ike-scan) is Copyright (C) 2003-2004 Roy Hills,
+ * The IKE Scanner (ike-scan) is Copyright (C) 2003-2005 Roy Hills,
  * NTA Monitor Ltd.
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: wrappers.c,v 1.3 2004/01/02 09:30:47 rsh Exp $
+ * $Id: wrappers.c,v 1.5 2005/01/01 16:49:53 rsh Exp $
  *
  * Author: Roy Hills
  * Date: 8 November 2003
@@ -41,7 +41,7 @@
 
 #include "ike-scan.h"
 
-static char rcsid[] = "$Id: wrappers.c,v 1.3 2004/01/02 09:30:47 rsh Exp $";	/* RCS ID for ident(1) */
+static char rcsid[] = "$Id: wrappers.c,v 1.5 2005/01/01 16:49:53 rsh Exp $";	/* RCS ID for ident(1) */
 
 /*
  * We omit the timezone arg from this wrapper since it's obsolete and we never
@@ -81,5 +81,5 @@ void *Realloc(void *ptr, size_t size) {
 }
 
 void wrappers_use_rcsid(void) {
-   printf("%s\n", rcsid);	/* Use rcsid to stop compiler optimising away */
+   fprintf(stderr, "%s\n", rcsid);	/* Use rcsid to stop compiler optimising away */
 }
